//
// VMime library (http://www.vmime.org)
// Copyright (C) 2002 Vincent Richard <vincent@vmime.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 3 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Linking this library statically or dynamically with other modules is making
// a combined work based on this library.  Thus, the terms and conditions of
// the GNU General Public License cover the whole combination.
//

#include "vmime/config.hpp"


#if VMIME_HAVE_MESSAGING_FEATURES


#include "vmime/security/defaultAuthenticator.hpp"

#include "vmime/net/service.hpp"

#include "vmime/platform.hpp"


namespace vmime {
namespace security {


defaultAuthenticator::defaultAuthenticator() {

}


defaultAuthenticator::~defaultAuthenticator() {

}


const string defaultAuthenticator::getUsername() const {

	shared_ptr <const net::service> service = m_service.lock();

	const string prefix = service->getInfos().getPropertyPrefix();
	const propertySet& props = service->getSession()->getProperties();

	if (props.hasProperty(prefix + net::serviceInfos::property::AUTH_USERNAME.getName())) {
		return props[prefix + net::serviceInfos::property::AUTH_USERNAME.getName()];
	}

	throw exceptions::no_auth_information();
}


const string defaultAuthenticator::getPassword() const {

	shared_ptr <const net::service> service = m_service.lock();

	const string prefix = service->getInfos().getPropertyPrefix();
	const propertySet& props = service->getSession()->getProperties();

	if (props.hasProperty(prefix + net::serviceInfos::property::AUTH_PASSWORD.getName())) {
		return props[prefix + net::serviceInfos::property::AUTH_PASSWORD.getName()];
	}

	throw exceptions::no_auth_information();
}


const string defaultAuthenticator::getAccessToken() const {

	shared_ptr <const net::service> service = m_service.lock();

	const string prefix = service->getInfos().getPropertyPrefix();
	const propertySet& props = service->getSession()->getProperties();

	if (props.hasProperty(prefix + net::serviceInfos::property::AUTH_ACCESS_TOKEN.getName())) {
		return props[prefix + net::serviceInfos::property::AUTH_ACCESS_TOKEN.getName()];
	}

	throw exceptions::no_auth_information();
}


const string defaultAuthenticator::getHostname() const {

	return platform::getHandler()->getHostName();
}


const string defaultAuthenticator::getAnonymousToken() const {

	return "anonymous@" + platform::getHandler()->getHostName();
}


const string defaultAuthenticator::getServiceName() const {

	// Information cannot be provided
	throw exceptions::no_auth_information();
}


void defaultAuthenticator::setService(const shared_ptr <net::service>& serv) {

	m_service = serv;
}


weak_ptr <net::service> defaultAuthenticator::getService() const {

	return m_service;
}


} // security
} // vmime


#endif // VMIME_HAVE_MESSAGING_FEATURES
